<?php

namespace FiloBlu\ExtInventory\Plugin\Api;

use Exception;
use FiloBlu\ExtInventory\Api\Data\InventoryFbOrderItemSourceInterfaceFactory;
use FiloBlu\ExtInventory\Api\InventoryFbOrderItemSourceRepositoryInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Sales\Api\Data\OrderItemExtensionInterfaceFactory;
use Magento\Sales\Api\Data\OrderItemInterface;
use Magento\Sales\Api\Data\OrderItemSearchResultInterface;

/**
 * Class OrderItemRepositoryInterface
 * @package FiloBlu\ExtInventory\Plugin\Api
 */
class OrderItemRepositoryInterface
{

    /**
     * @var InventoryFbOrderItemSourceRepositoryInterface
     */
    private $inventoryFbOrderItemSourceRepository;

    /**
     * @var OrderItemExtensionInterfaceFactory
     */
    private $orderItemExtensionInterfaceFactory;
    /**
     * @var InventoryFbOrderItemSourceInterfaceFactory
     */
    private $fbOrderItemSourceInterfaceFactory;

    /**
     * OrderItemRepositoryInterface constructor.
     * @param InventoryFbOrderItemSourceRepositoryInterface $inventoryFbOrderItemSourceRepository
     * @param OrderItemExtensionInterfaceFactory $orderItemExtensionInterfaceFactory
     * @param InventoryFbOrderItemSourceInterfaceFactory $fbOrderItemSourceInterfaceFactory
     */
    public function __construct(
        InventoryFbOrderItemSourceRepositoryInterface $inventoryFbOrderItemSourceRepository,
        OrderItemExtensionInterfaceFactory $orderItemExtensionInterfaceFactory,
        InventoryFbOrderItemSourceInterfaceFactory $fbOrderItemSourceInterfaceFactory
    ) {
        $this->inventoryFbOrderItemSourceRepository = $inventoryFbOrderItemSourceRepository;
        $this->orderItemExtensionInterfaceFactory = $orderItemExtensionInterfaceFactory;
        $this->fbOrderItemSourceInterfaceFactory = $fbOrderItemSourceInterfaceFactory;
    }

    /**
     * @param \Magento\Sales\Api\OrderItemRepositoryInterface $subject
     * @param OrderItemSearchResultInterface $searchCriteria
     * @return OrderItemSearchResultInterface
     */
    public function afterGetList(\Magento\Sales\Api\OrderItemRepositoryInterface $subject, OrderItemSearchResultInterface $searchCriteria)
    {
        foreach ($searchCriteria->getItems() as $entity) {
            $extensionAttributes = $entity->getExtensionAttributes();
            if ($extensionAttributes === null) {
                $extensionAttributes = $this->orderItemExtensionInterfaceFactory->create();
            }

            try {
                $filoBluOrderItem = $this->inventoryFbOrderItemSourceRepository->getByOrderItem($entity);
                $extensionAttributes->setFilobluOrderItem($filoBluOrderItem);
            } catch (Exception $exception) {
                $extensionAttributes->setFilobluOrderItem($this->fbOrderItemSourceInterfaceFactory->create());
            }
            $entity->setExtensionAttributes($extensionAttributes);
        }
        return $searchCriteria;
    }

    /**
     * @param \Magento\Sales\Api\OrderItemRepositoryInterface $subject
     * @param OrderItemInterface $result
     * @param int $id
     * @return OrderItemInterface
     */

    public function afterGet(\Magento\Sales\Api\OrderItemRepositoryInterface $subject, $result, $id)
    {
        $extensionAttributes = $result->getExtensionAttributes();
        if ($extensionAttributes === null) {
            $extensionAttributes = $this->orderItemExtensionInterfaceFactory->create();
        }
        try {
            $filoBluOrderItem = $this->inventoryFbOrderItemSourceRepository->get($id);
            $extensionAttributes->setFilobluOrderItem($filoBluOrderItem);
        } catch (Exception $exception) {
            $extensionAttributes->setFilobluOrderItem($this->fbOrderItemSourceInterfaceFactory->create());
        }
        $result->setExtensionAttributes($extensionAttributes);

        return $result;
    }

    /**
     * @param \Magento\Sales\Api\OrderItemRepositoryInterface $subject
     * @param OrderItemInterface $entity
     * @return OrderItemInterface
     * @throws LocalizedException
     */
    public function afterSave(
        \Magento\Sales\Api\OrderItemRepositoryInterface $subject,
        OrderItemInterface $entity
    ) {
        $extensionAttributes = $entity->getExtensionAttributes();

        if ($extensionAttributes === null) {
            return $entity;
        }

        $filoBluOrderItem = $extensionAttributes->getFilobluOrderItem();

        if ($filoBluOrderItem === null) {
            return $entity;
        }

        $this->inventoryFbOrderItemSourceRepository->save($filoBluOrderItem);

        return $entity;
    }
}
